package mqtt.publisher;

import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Collections;
import java.util.Random;

import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.MqttSecurityException;
import org.eclipse.paho.client.mqttv3.MqttTopic;

public class MqttPublisher {

	private static final String BROKER_URL = "tcp://broker.mqttdashboard.com:1883";

	private static final String TOPIC_TEMPERATURE = "sensordevice/temperature";

	/**
	 * @return random temperature value from 10 to 30 degree.
	 */
	private static String getTemperature() {
		Random random = new Random();
		return String.valueOf(random.nextInt(10) + 20);
	}

	public static void main(String... args) throws MqttSecurityException,
			MqttException, SocketException {
		String clientId = getMacAddress() + "-temperature";
		// 1. Create a MQTT client instance.
		MqttClient client = new MqttClient(BROKER_URL, clientId);
		// 2. Start temperature publishing.
		start(client);
	}

	private static void start(MqttClient client) throws MqttSecurityException,
			MqttException {
		try {
			// 3. Setup a new connection.
			MqttConnectOptions options = new MqttConnectOptions();
			options.setCleanSession(true);
			client.connect(options);
			while (true) {
				// 4. Publish temperature every second.
				publishTemperature(client);
				Thread.sleep(1000);
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private static void publishTemperature(MqttClient client)
			throws MqttException {
		// 5. Get a temperature topic.
		MqttTopic temperatureTopic = client.getTopic(TOPIC_TEMPERATURE);
		// 4. Create a new random temperature value.
		String temperature = getTemperature();
		// 6. Publish a new value to the broker.
		temperatureTopic.publish(new MqttMessage(temperature.getBytes()));
		System.out.println("published: " + temperature);
	}

	/**
	 * Get MAC address of a current system.
	 * 
	 * @return MAC address
	 * @throws SocketException
	 */
	public static String getMacAddress() throws SocketException {
		StringBuilder result = new StringBuilder();
		for (NetworkInterface ni : Collections.list(NetworkInterface
				.getNetworkInterfaces())) {
			byte[] hardwareAddress = ni.getHardwareAddress();
			if (hardwareAddress != null) {
				for (int i = 0; i < hardwareAddress.length; i++) {
					result.append(String.format((i == 0 ? "" : "-") + "%02X",
							hardwareAddress[i]));
				}
				break;
			}
		}
		return result.toString();
	}

}
